"""Internal helper functions"""

import os.path
import datetime
from configparser import ConfigParser
from subprocess import check_output

import click
from click import MissingParameter

PKG_DIR = os.path.abspath(os.path.dirname(__file__))
TEMPLATES_DIR = os.path.join(PKG_DIR, 'templates')


def _parse_list_option(opt):
    """Split list element separated by a comma."""
    return opt.split(',')


def _get_username():
    try:
        name = check_output(
            ['git', 'config', '--get', 'user.name']).decode()[:-1]
    except:
        name = ''

    return name


def _get_usermail():
    try:
        email = check_output(
            ['git', 'config', '--get', 'user.email']).decode()[:-1]
    except:
        email = ''

    return email


def _check_riotbase(path):
    """Check the given path is a valid RIOTBASE directory."""
    coc = os.path.join(os.path.expanduser(path), 'CODE_OF_CONDUCT.md')
    if os.path.isfile(coc):
        first_line = open(coc, 'r').readline()[:-1]
        if first_line == 'RIOT-OS Code of Conduct':
            return os.path.expanduser(path)
    raise MissingParameter(param_type='RIOT base directory')


def _check_common_params(params):
    if 'year' not in params:
        params['year'] = datetime.datetime.now().year
    if 'author_name' not in params:
        params['author_name'] = _get_username()
    if 'author_email' not in params:
        params['author_email'] = _get_usermail()
    if 'organization' not in params:
        params['organization'] = _get_username()
    if 'riotbase' not in params:
        params['riotbase'] = ''
    params['riotbase'] = _check_riotbase(params['riotbase'])


def _prompt_common_information():
    params = {}
    params['year'] = datetime.datetime.now().year
    params['author_name'] = click.prompt(
        text='Author name', default=_get_username())
    params['author_email'] = click.prompt(
        text='Author email', default=_get_usermail())
    params['organization'] = click.prompt(
        text='Organization', default=_get_username())
    params['riotbase'] = click.prompt(
        text='RIOT base directory', value_proc=_check_riotbase)
    return params


def _read_config(filename, section=None):
    parser = ConfigParser()
    parser.readfp(filename)
    config = dict(parser.items('common'))
    if section is not None:
        config.update(dict(parser.items(section)))
    return config


def write_application_source(output_dir, params, template_dir='application'):
    app_dir = os.path.join(TEMPLATES_DIR, template_dir)
    files = {os.path.join(app_dir, f_name): os.path.join(output_dir, f_name)
             for f_name in ['main.c', 'Makefile', 'README.md']}

    for file_in, file_out in files.items():
        with open(file_in, 'r') as f_in:
            with open(file_out, 'w') as f_out:
                f_out.write(f_in.read().format(**params))


"""RIOT application generator module."""

import os.path
import datetime

import click
from click import MissingParameter

from .helpers import _get_usermail, _get_username
from .helpers import TEMPLATES_DIR
from .helpers import _read_config, _parse_list_option
from .helpers import _prompt_common_information, _check_common_params
from .helpers import write_application_source


def _read_application_config(filename):
    """Read the application specific configuration file."""
    params = _read_config(filename, section='application')
    if 'name' not in params or not params['name']:
        raise MissingParameter(param_type='application name')
    if 'brief' not in params:
        params['brief'] = ''
    if 'board' not in params:
        params['board'] = 'native'
    else:
        params['board'] = params['board']
    for param in ['modules', 'packages', 'features']:
        if param not in params:
            params[param] = ''
        else:
            params[param] = _parse_list_option(params[param])
    return params


def _prompt_application_params():
    """Request application specific variables."""
    params = {}
    params['name'] = click.prompt(
        text='Application name (no space)')
    params['brief'] = click.prompt(
        text='Application brief description', default='')
    params['board'] = click.prompt(text='Target board', default='native')
    params['modules'] = click.prompt(
        text='Required modules (comma separated)', default='',
        value_proc=_parse_list_option)
    params['packages'] = click.prompt(
        text='Required packages (comma separated)', default='',
        value_proc=_parse_list_option)
    params['features'] = click.prompt(
        text='Required board features (comma separated)', default='',
        value_proc=_parse_list_option)

    params.update(_prompt_common_information())
    return params


def _check_application_params(params):
    application_name = params['name'].replace(' ', '_')
    params['name'] = application_name
    params['name_underline'] = '=' * len(application_name)
    params['includes'] = ''
    for module in params['modules']:
        params['includes'] += 'USEMODULE += {}\n'.format(module)
    for package in params['packages']:
        params['includes'] += 'USEPKG += {}\n'.format(package)
    for feature in params['features']:
        params['includes'] += 'FEATURES_REQUIRED += {}\n'.format(feature)


@click.command()
@click.argument('output_dir', type=click.Path(exists=True))
@click.option('--config', type=click.File(mode='r'))
def application(output_dir, config):
    # Use config file if it's set
    if config is not None:
        params = _read_application_config(config)
    else:
        params = _prompt_application_params()
    _check_application_params(params)
    _check_common_params(params)
    output_dir = os.path.expanduser(output_dir)
    write_application_source(output_dir, params)

    click.echo(click.style('Application \'{name}\' generated with success!'
                           .format(**params), bold=True))
    click.echo('\nTo build the application, use')
    click.echo('\n     make -C {}\n'.format(output_dir))



import logging
import click

from .application import application
from .board import board
from .test import test

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s - %(name)14s - '
                           '%(levelname)5s - %(message)s')
logger = logging.getLogger("riotgen.generator")


@click.group()
def cli():
    pass

# register subcommands
cli.add_command(application)
cli.add_command(board)
cli.add_command(test)


"""RIOT application generator module."""

import os.path
import datetime

import click
from click import MissingParameter

from .helpers import _get_usermail, _get_username
from .helpers import TEMPLATES_DIR
from .helpers import _read_config, _parse_list_option
from .helpers import _prompt_common_information, _check_common_params


def _read_board_config(filename):
    """Read the board specific configuration file."""
    params = _read_config(filename, section='board')
    if 'name' not in params or not params['name']:
        raise MissingParameter(param_type='board name')
    if 'displayed_name' not in params or not params['displayed_name']:
        raise MissingParameter(param_type='board displayed name')
    if 'cpu' not in params:
        raise MissingParameter(param_type='cpu name')
    if 'cpu_model' not in params:
        raise MissingParameter(param_type='cpu model name')
    if 'description' not in params:
        params['description'] = ''
    if 'features' not in params:
        params['features'] = ''
    else:
        params['features'] = _parse_list_option(params['features'])
    return params


def _prompt_board_params():
    """Request application specific variables."""
    params = {}
    params['name'] = click.prompt(
        text='Board name (no space)')
    params['displayed_name'] = click.prompt(
        text='Board displayed name (for doxygen documentation)')
    params['cpu'] = click.prompt(
        text='CPU name')
    params['cpu_model'] = click.prompt(
        text='CPU model name')
    params['description'] = click.prompt(
        text='Application detailed description', default='')
    params['features'] = click.prompt(
        text='Features provided by this board (comma separated)', default='',
        value_proc=_parse_list_option)

    params.update(_prompt_common_information())
    return params


def _check_board_params(params):
    board_name = params['name'].replace(' ', '_')
    params['name'] = board_name
    params['features_provided'] = ''
    for feature in sorted(params['features']):
        params['features_provided'] += \
            'FEATURES_PROVIDED += periph_{}\n'.format(feature)


@click.command()
@click.option('--config', type=click.File(mode='r'))
def board(config):
    # Use config file if it's set
    if config is not None:
        params = _read_board_config(config)
    else:
        params = _prompt_board_params()
    _check_board_params(params)
    _check_common_params(params)

    tpl_dir = os.path.join(TEMPLATES_DIR, 'board')
    boards_dir = os.path.join(os.path.expanduser(params['riotbase']), 'boards')
    board_dir = os.path.join(boards_dir, params['name'])
    board_include_dir = os.path.join(board_dir, 'include')
    if not os.path.exists(board_dir):
        os.makedirs(board_dir)
        os.makedirs(board_include_dir)
    elif not click.prompt('\'{name}\' board directory already exists, '
                          'overwrite (y/N)?'.format(**params),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    files = {os.path.join(tpl_dir, f_name): os.path.join(board_dir, f_name)
             for f_name in ['board.c', 'doc.txt', 'Makefile', 'Makefile.dep',
                            'Makefile.features', 'Makefile.include']}
    files.update({os.path.join(tpl_dir, f_name):
                               os.path.join(board_include_dir, f_name)
                  for f_name in ['board.h', 'periph_conf.h']})

    for file_in, file_out in files.items():
        with open(file_in, 'r') as f_in:
            with open(file_out, 'w') as f_out:
                f_out.write(f_in.read().format(**params))

    click.echo(click.style('Board support generated!', bold=True))



"""RIOT test generator module."""

import os.path
import datetime

import click
from click import MissingParameter

from .helpers import _get_usermail, _get_username
from .helpers import TEMPLATES_DIR
from .helpers import _read_config, _parse_list_option
from .helpers import _prompt_common_information, _check_common_params
from .helpers import write_application_source


def _read_test_config(filename):
    """Read the test specific configuration file."""
    params = _read_config(filename, section='test')
    if 'name' not in params or not params['name']:
        raise MissingParameter(param_type='test name')
    if 'brief' not in params:
        params['brief'] = ''
    if 'board' not in params:
        params['board'] = 'native'
    else:
        params['board'] = params['board']
    for param in ['modules', 'packages', 'features']:
        if param not in params:
            params[param] = ''
        else:
            params[param] = _parse_list_option(params[param])
    return params


def _prompt_test_params():
    """Request test specific variables."""
    params = {}
    params['name'] = click.prompt(
        text='Application name (no space)')
    params['brief'] = click.prompt(
        text='Test brief description', default='')
    params['board'] = click.prompt(text='Target board', default='native')
    params['modules'] = click.prompt(
        text='Required modules (comma separated)', default='',
        value_proc=_parse_list_option)
    params['packages'] = click.prompt(
        text='Required packages (comma separated)', default='',
        value_proc=_parse_list_option)
    params['features'] = click.prompt(
        text='Required board features (comma separated)', default='',
        value_proc=_parse_list_option)

    params.update(_prompt_common_information())
    return params


def _check_test_params(params):
    test_name = params['name'].replace(' ', '_')
    params['name'] = test_name
    params['includes'] = ''
    for module in params['modules']:
        params['includes'] += 'USEMODULE += {}\n'.format(module)
    for package in params['packages']:
        params['includes'] += 'USEPKG += {}\n'.format(package)
    for feature in params['features']:
        params['includes'] += 'FEATURES_REQUIRED += {}\n'.format(feature)


@click.command()
@click.option('--config', type=click.File(mode='r'))
def test(config):
    # Use config file if it's set
    if config is not None:
        params = _read_test_config(config)
    else:
        params = _prompt_test_params()
    _check_test_params(params)
    _check_common_params(params)

    tests_dir = os.path.join(os.path.expanduser(params['riotbase']), 'tests')
    test_dir = os.path.join(tests_dir, params['name'])
    if not os.path.exists(test_dir):
        os.makedirs(test_dir)
    elif not click.prompt('\'{name}\' test directory already exists, '
                          'overwrite (y/N)?'.format(**params),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    output_dir = os.path.expanduser(test_dir)
    write_application_source(output_dir, params, template_dir='test')

    click.echo(click.style('Test application \'{name}\' generated!'
                           .format(**params), bold=True))



"""RIOT generator is a helper library for bootstraping RIOT code.

It can be used to generate:
- a new board support with the default structure
- a new driver support
- a new application
- a new test
"""

__version__ = '0.1.1'

from .generator import cli
